﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace StuderInterface.Helpers
{
    public static class Converters
    {
        /// <summary>
        /// 4bytes as UInt32 convert to float value
        /// IEE754
        /// litle endian
        /// </summary>
        /// <param name="f"></param>
        /// <returns></returns>
        public static float FromFloatSafe(IEnumerable<byte> fourByte)
        {
            if (fourByte == null || fourByte.Count() < 4)
                return 0;

            uint fb = 0;
            //higher is most right byte
            for (int i = 3; i >= 0; i--)
            {
                fb = fb + fourByte.ElementAt(i);
                if (i > 0)
                    fb = fb << 8;
            }

            int sign = (int)((fb >> 31) & 1);
            int exponent = (int)((fb >> 23) & 0xFF);
            int mantissa = (int)(fb & 0x7FFFFF);

            float fMantissa;
            float fSign = sign == 0 ? 1.0f : -1.0f;

            if (exponent != 0)
            {
                exponent -= 127;
                fMantissa = 1.0f + (mantissa / (float)0x800000);
            }
            else
            {
                if (mantissa != 0)
                {
                    // denormal
                    exponent -= 126;
                    fMantissa = 1.0f / (float)0x800000;
                }
                else
                {
                    // +0 and -0 cases
                    fMantissa = 0;
                }
            }

            float fExponent = (float)Math.Pow(2.0, exponent);
            float ret = fSign * fMantissa * fExponent;
            return ret;
        }
    }
}
