﻿using StuderInterface.Helpers;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Text.Json;
using System.Text.Json.Serialization;
using System.Threading.Tasks;

namespace StuderInterface.HttpClient
{
    public class HttpSender
    {
        #region members
        /// <summary>
        /// should be one instance per application
        /// </summary>
        private System.Net.Http.HttpClient client;
        /// <summary>
        /// timer for sending data in given interval
        /// </summary>
        System.Timers.Timer sendTimer;
        List<ValueItem> valueList;
        object listlock = new object();
        bool debugOn;
        #endregion

        #region constructors
        /// <summary>
        /// "http://www.mms-softec.sk/PIP/Test.aspx/NewData?d=abcdefgh"
        /// </summary>
        /// <param name="serverUrl"></param>
        /// <param name="period">interval to send data in seconds, default 10s</param>
        public HttpSender(string serverUrl, bool debugOn)
        {
            ConstructorHttpSender(serverUrl, 10, debugOn);
        }
        /// <summary>
        /// "http://www.mms-softec.sk/PIP/Test.aspx/NewData?d=abcdefgh"
        /// </summary>
        /// <param name="serverUrl"></param>
        /// <param name="period">interval to send data in seconds, default 10s</param>
        public HttpSender(string serverUrl, int period, bool debugOn)
        {
            ConstructorHttpSender(serverUrl, period, debugOn);
        }
            /// <summary>
            /// "http://www.mms-softec.sk/PIP/Test.aspx/NewData?d=abcdefgh"
            /// </summary>
            /// <param name="serverUrl"></param>
            /// <param name="period">interval to send data in seconds, default 10s</param>
        public void ConstructorHttpSender(string serverUrl, int period, bool debugOn)
        {
            valueList = new List<ValueItem>();
            sendTimer = new System.Timers.Timer(period * 1000);
            sendTimer.Enabled = false;
            sendTimer.AutoReset = true;
            sendTimer.Elapsed += SendTimer_Elapsed;
            this.debugOn = debugOn;
            client = new System.Net.Http.HttpClient();
            client.BaseAddress = new Uri(serverUrl);//"http://www.mms-softec.sk/PIP/Test.aspx/NewData?d=abcdefgh"
            client.DefaultRequestHeaders.Accept.Clear();
            client.DefaultRequestHeaders.Accept.Add(
                new MediaTypeWithQualityHeaderValue("application/json"));
        }
        #endregion

        #region properties
        public bool IsSendingEnabled
        {
            get { return sendTimer.Enabled; }
            set { sendTimer.Enabled = value; }
        }
        #endregion

        #region methods
        /// <summary>
        /// add values in internal buffer, they are averaged for send interval
        /// </summary>
        public void AddValues(uint addr, UInt32 objectId, object value)
        {
            lock (listlock)
            {
                valueList.Add(new ValueItem() { Addr = addr, ObjectId = objectId, Value = value });
            }
        }

        #endregion

        #region events
        /// <summary>
        /// it's time to send data,
        /// 1. calculate averages, build UnitData and send them in one post to server
        /// to have data from parameters scanned with longer period, still keeps last data item for each params.
        /// If there are new data it is ignored otherwise is ised historical item
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SendTimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            try
            {
                sendTimer.Stop(); //this makes debugging easier
                List<UnitData> unitDataList = new List<UnitData>();
                List<ValueItem> aggrList = new List<ValueItem>();

                lock (listlock)
                {
                    var grpFloat = valueList.Where(a => a.Value is float).GroupBy(a => new { a.Addr, a.ObjectId }).ToList();
                    // take average from new items, if not take historical (marked processed)
                    var averagedFloatValList = grpFloat.Select(a => new ValueItem()
                    {
                        Addr = a.Key.Addr,
                        ObjectId = a.Key.ObjectId,
                        Value = (a.Count(b => !b.Processed) > 0) ? a.Where(b => !b.Processed).Average(b => (float)b.Value) : a.Select(b => b.Value).FirstOrDefault()
                    });
                    aggrList.AddRange(averagedFloatValList);
                    //take last flag, or previous value when not ready
                    var grpUInt16 = valueList.Where(a => a.Value is UInt16).GroupBy(a => new { a.Addr, a.ObjectId });
                    var lastFlags = grpUInt16.Select(a => new ValueItem()
                    {
                        Addr = a.Key.Addr,
                        ObjectId = a.Key.ObjectId,
                        Value = a.Select(b => b.Value).LastOrDefault()
                    });
                    aggrList.AddRange(lastFlags);
                    //delete items in unitDataList
                    var grps = valueList.GroupBy(a => new { a.Addr, a.ObjectId }).ToList();
                    foreach (var grp in grps)
                    {
                        var toRemove = valueList.Where(a => a.Addr == grp.Key.Addr && a.ObjectId == grp.Key.ObjectId).SkipLast(1).ToList();
                        foreach (var item in toRemove)
                        {
                            valueList.Remove(item);
                        }
                    }
                    //mark remaining items in valueList as processed
                    foreach (var item in valueList)
                    {
                        item.Processed = true;
                    }
                }
                //build JSON object
                //group by address
                 var addrGrps = aggrList.GroupBy(a => a.Addr);
                foreach (var item in addrGrps)
                {
                    //XTM, BSP, .... unit with one address
                    var unitData = new UnitData(item.Key);
                    unitDataList.Add(unitData);
                    //add parameters with data
                    foreach (var data in item)
                    {
                        unitData.AddData(data.ObjectId, data.Value);
                    }
                }
                //serialize
                StuderDataSet sds = new StuderDataSet() { UnitDataList = unitDataList.ToList() };
                string jsonStr = JsonSerializer.Serialize(sds);
                //send
                _= AsyncPostData(jsonStr);
            }
            catch(Exception ex)
            {
                //logging??
                //Console.WriteLine(ex.Message);
                Serilog.Log.Error(ex.Message);
            }
            finally
            {
                sendTimer.Start();
            }
        }
        #endregion
        /// <summary>
        /// because sometimes problem with desrialize in webmethod send it as raw byte array
        /// </summary>
        /// <param name="addr"></param>
        /// <param name="ObjectId"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        async Task AsyncPostData(string jsonString)
        {
            try
            {
                //I need to send it as {'items':[{'Addr':101,'Data':[{'ObjectId':3000,'Value':50.25}]}]}"
                //where receiving with WebMethod AddData(object items)
                //string str = "{ 'items':[{ 'Addr':101,'Data':[{ 'ObjectId':3000,'Value':50.25}]}]}";
                //string str = string.Format("{{'items':{0}}}", jsonString);//"{'items':" + jsonString + "}";

                //var content = new StringContent(jsonString, Encoding.UTF8, "application/json");
                var content = new StringContent(jsonString, Encoding.UTF8, "text/plain");
                HttpResponseMessage response = await client.PostAsync("", content);
                if (!response.IsSuccessStatusCode)
                {
                    //Console.WriteLine("Web server: " + response.StatusCode.ToString());
                    Serilog.Log.Error("Web server: " + response.StatusCode.ToString());
                }
                //var resultConten = await response.Content.ReadAsStringAsync();
                // return result of web server.
                //return resultConten;
            }
            catch(Exception ex)
            {
                //logging??
                //Console.WriteLine("Web server: " + ex.Message);
                Serilog.Log.Error("Web server: " + ex.Message);
            }
        }
    }

    internal class ValueItem
    {
        public UInt32 Addr { get; set; }
        public UInt32 ObjectId { get; set; }
        public object Value { get; set; }
        public bool Processed { get; set; }
    }
}
