﻿using System;
using System.Linq;
using Microsoft.Extensions.Configuration;
using Serilog;
using Studer;
using StuderEmulator.Studer;
using StuderInterface.Helpers;
using StuderInterface.HttpClient;
using StuderInterface.Settings;
using StuderInterface.Studer;

namespace StuderInterface
{
    class Program
    {
        static void Main(string[] args)
        {
            HttpSender httpSender = null;
            Log.Logger = new LoggerConfiguration()
                .MinimumLevel.Debug()
                .WriteTo.Console()
                .WriteTo.File("logs\\interface_log.txt", rollingInterval: RollingInterval.Day)
                .CreateLogger();
            try
            {
                SISettings sisettings = null;
                Log.Information("Interface starting");
                //Console.WriteLine("Interface starting");
                try
                {
                    IConfigurationRoot configuration = new ConfigurationBuilder().AddJsonFile("settings.json").Build();
                    sisettings = configuration.Get<SISettings>();
                    if (sisettings != null)
                    {
                        //Console.WriteLine(string.Format("XCom connecting on port: {0}", sisettings.PortName));
                        Log.Information(string.Format("XCom connecting on port: {0}", sisettings.PortName));
                        if (!string.IsNullOrWhiteSpace(sisettings.WebUrl))
                        {
                            //webApi sender
                            //Console.WriteLine(string.Format("Using WEB Api on: {0}", sisettings.WebUrl));
                            Log.Information(string.Format("Using WEB Api on: {0}", sisettings.WebUrl));
                        }
                    }
                }
                catch(Exception ex)
                {
                    throw(new Exception("Error in settings.json :" + ExceptionHelper.GetExceptionMessages(ex)));
                }

                StuderComunicator comunicator = new StuderComunicator(sisettings, new Action<string>((msg) =>
                {
                    //Console.WriteLine(msg);
                    Log.Information(msg);
                }));
                comunicator.OnDataReceived += new StuderComunicator.DataReceived(new Action<object, object>((sender, e) => 
                    {
                        if (e is ErrResponse)
                        {
                            var err = e as ErrResponse;
                            //Console.WriteLine("Error: target {0} {1}", err.ReqItem?.Addr, err.ErrMsg);
                            Log.Error(string.Format("Error: target {0}-{1} {2}", err.ReqItem?.Addr, err.ReqItem?.ObjectId, err.ErrMsg));
                        }
                        else
                        if (e is Frame)
                        {
                            var frame = e as Frame;
                            if (sisettings.WriteDataToConsole)
                            {
                                //send data somewhere
                                //Console.WriteLine("Data\n:{0}", string.Join(" ", frame.GetBytes().ToList().Select(a => a.ToString("X2")).ToArray()));
                                Log.Information("Data\n:{0}", string.Join(" ", frame.GetBytes().ToList().Select(a => a.ToString("X2")).ToArray()));
                                //Console.WriteLine("Addr: {0}", frame.Header.SrcAddr);
                                Log.Information("Addr: {0}", frame.Header.SrcAddr);
                                //Console.WriteLine("ObjectId: {0}", frame.ObjectId);
                                Log.Information("ObjectId: {0}", frame.ObjectId);
                                FormatEnum format = sisettings.GetStuderFormat(frame.Header.SrcAddr, (UInt16)frame.ObjectType, frame.ObjectId);
                                var value = StuderConverter.ValueToCsObject(format, frame.PropertyData);
                                //Console.WriteLine("Value: {0}", value);
                                Log.Information("Value: {0}", value);
                            }

                            if (httpSender != null)
                            {
                                if (frame.IsError)
                                {

                                }
                                else
                                {
                                    //convert bytes in frame.PropertyData to variable of correct data type
                                    //debug
                                    //comunicator.Stop();

                                    FormatEnum format = sisettings.GetStuderFormat(frame.Header.SrcAddr, (UInt16)frame.ObjectType, frame.ObjectId);
                                    var value = StuderConverter.ValueToCsObject(format, frame.PropertyData);
                                    httpSender.AddValues(frame.Header.SrcAddr, frame.ObjectId, value);
                                }
                            }
                        }
                    }));
                if (!string.IsNullOrWhiteSpace(sisettings.WebUrl))
                {
                    //webApi sender
                    httpSender = new HttpSender(sisettings.WebUrl, 10, sisettings.Debug);
                    
                }

                comunicator.Start();
                if (httpSender != null)
                    httpSender.IsSendingEnabled = true;

                //Console.WriteLine("Interface is running.");
                Log.Information("Interface is running.");

                Console.ReadLine();
                comunicator.Stop();
            }
            catch (Exception ex)
            {
                //Console.WriteLine("Interface exception: " + ex?.Message);
                Log.Error("Interface exception: " + ex?.Message);
            }
            finally
            {
                
                Log.Information("Interface stopped.");
                Console.WriteLine("Press Enter to close.");
                //debug
                Console.ReadLine();
            }
        }
    }
}
