﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Studer
{
    public enum ObjectTypeEnum: UInt16
    {
        UNDEFINED = 0x00,
        /// <summary>
        /// These objects are the information about the current state of the system. 
        /// They cannot be modified and their values change during the operation of the system. Previously known as system states.
        /// </summary>
        USER_INFO_OBJECT_TYPE = 0x01,
        /// <summary>
        /// All parameters accessible from the remote control can also be modified with the protocol.
        /// The behaviour is the same as if a physical person changes the value with the remote control buttons. 
        /// Currently, only changes at the level qsp are possible.
        /// Values of type FLOAT are stored internally in various 16 bit fixed point formats. 
        /// For this reason, the read back value after a write can be rounded slightly.
        /// </summary>
        PARAMETER_OBJECT_TYPE = 0x02,

        MESSAGE_OBJECT_TYPE = 0x03,
        /// <summary>
        /// The custom datalog field is a string present in each Xcom-232i on the system. 
        /// It has a maximum length of 250 bytes and is stored in volatile memory.
        /// </summary>
        CUSTOM_DATALOG_FIELD_OBJECT = 0x05
    }
}
